package Nour;
# ABSTRACT: This module does nothing at the moment.

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Nour - This module does nothing at the moment.

=head1 VERSION

version 0.04

=head1 DESCRIPTION

Temporary placeholder for something useful, maybe...

=head1 NAME

Nour

=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan

=head1 SUPPORT

=head2 Bugs / Feature Requests

Please report any bugs or feature requests through the issue tracker
at L<https://github.com/sharabash/nour/issues>.
You will be notified automatically of any progress on your issue.

=head2 Source Code

This is open source software.  The code repository is available for
public review and contribution under the terms of the license.

L<https://github.com/sharabash/nour>

  git clone git://github.com/sharabash/nour.git

=head1 AUTHOR

Nour Sharabash <amirite@cpan.org>

=head1 CONTRIBUTOR

Nour Sharabash <nour.sharabash@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Nour Sharabash.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
